@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,15 +26,21 @@
 #pragma interface                       /* gcc class implementation */
 #endif
 
+/* DDL names have to fit in system table ndb_schema */
+#define NDB_MAX_DDL_NAME_BYTESIZE 63
+#define NDB_MAX_DDL_NAME_BYTESIZE_STR "63"
+
 /* Blob tables and events are internal to NDB and must never be accessed */
 #define IS_NDB_BLOB_PREFIX(A) is_prefix(A, "NDB$BLOB")
 
-#include <NdbApi.hpp>
-#include <ndbapi_limits.h>
+#include <ndbapi/NdbApi.hpp>
+#include <ndbapi/ndbapi_limits.h>
+#include <kernel/ndb_limits.h>
+#include "ndb_conflict.h"
+
+#define NDB_IGNORE_VALUE(x) (void)x
 
 #define NDB_HIDDEN_PRIMARY_KEY_LENGTH 8
-#define NDB_DEFAULT_AUTO_PREFETCH 32
-
 
 class Ndb;             // Forward declaration
 class NdbOperation;    // Forward declaration
@@ -45,16 +51,9 @@ class NdbIndexScanOperation;
 class NdbBlob;
 class NdbIndexStat;
 class NdbEventOperation;
-class NdbInterpretedCode;
 class ha_ndbcluster_cond;
 class Ndb_event_data;
 
-// connectstring to cluster if given by mysqld
-extern const char *ndbcluster_connectstring;
-extern ulong ndb_cache_check_time;
-extern ulong ndb_report_thresh_binlog_epoch_slip;
-extern ulong ndb_report_thresh_binlog_mem_usage;
-
 typedef enum ndb_index_type {
   UNDEFINED_INDEX = 0,
   PRIMARY_KEY_INDEX = 1,
@@ -77,12 +76,6 @@ typedef struct ndb_index_data {
   const NdbDictionary::Index *unique_index;
   unsigned char *unique_index_attrid_map;
   bool null_in_unique_index;
-  // In this version stats are not shared between threads
-  NdbIndexStat* index_stat;
-  uint index_stat_cache_entries;
-  // Simple counter mechanism to decide when to connect to db
-  uint index_stat_update_freq;
-  uint index_stat_query_count;
   /*
     In mysqld, keys and rows are stored differently (using KEY_PART_INFO for
     keys and Field for rows).
@@ -132,47 +125,19 @@ typedef enum {
   NSS_ALTERED 
 } NDB_SHARE_STATE;
 
-#ifdef HAVE_NDB_BINLOG
-enum enum_conflict_fn_type
-{
-  CFT_NDB_UNDEF = 0
-  ,CFT_NDB_MAX
-  ,CFT_NDB_OLD
-  ,CFT_NDB_MAX_DEL_WIN
-};
-
-/* NdbOperation custom data which points out handler and record. */
-struct Ndb_exceptions_data {
-  struct st_ndbcluster_share *share;
-  const uchar* row;
-};
-
-typedef struct st_ndbcluster_conflict_fn_share {
-  enum_conflict_fn_type m_resolve_cft;
-
-  /* info about original table */
-  uint8 m_pk_cols;
-  uint8 m_resolve_column;
-  uint8 m_resolve_size;
-  uint8 unused;
-  uint16 m_offset[16];
-
-  const NdbDictionary::Table *m_ex_tab;
-  uint32 m_count;
-} NDB_CONFLICT_FN_SHARE;
-#endif
-
 /*
   Stats that can be retrieved from ndb
 */
 struct Ndb_statistics {
   Uint64 row_count;
   Uint64 commit_count;
-  Uint64 row_size;
+  ulong row_size;
   Uint64 fragment_memory;
+  Uint64 fragment_extent_space; 
+  Uint64 fragment_extent_free_space;
 };
 
-typedef struct st_ndbcluster_share {
+struct NDB_SHARE {
   NDB_SHARE_STATE state;
   MEM_ROOT mem_root;
   THR_LOCK lock;
@@ -187,6 +152,7 @@ typedef struct st_ndbcluster_share {
   char *table_name;
   Ndb::TupleIdRange tuple_id_range;
   struct Ndb_statistics stat;
+  struct Ndb_index_stat *index_stat_list;
   bool util_thread; // if opened by util thread
   uint32 connect_count;
   uint32 flags;
@@ -198,7 +164,7 @@ typedef struct st_ndbcluster_share {
   char *old_names; // for rename table
   MY_BITMAP *subscriber_bitmap;
   NdbEventOperation *new_op;
-} NDB_SHARE;
+};
 
 inline
 NDB_SHARE_STATE
@@ -277,11 +243,6 @@ inline void set_binlog_use_update(NDB_SHARE *share)
 inline my_bool get_binlog_use_update(NDB_SHARE *share)
 { return (share->flags & NSF_BINLOG_USE_UPDATE) != 0; }
 
-typedef enum ndb_query_state_bits {
-  NDB_QUERY_NORMAL = 0,
-  NDB_QUERY_MULTI_READ_RANGE = 1
-} NDB_QUERY_STATE_BITS;
-
 /*
   Place holder for ha_ndbcluster thread specific data
 */
@@ -293,14 +254,19 @@ enum THD_NDB_OPTIONS
     In participating mysqld, do not try to acquire global schema
     lock, as one other mysqld already has the lock.
   */
-  TNO_NO_LOCK_SCHEMA_OP= 1 << 1
+  TNO_NO_LOCK_SCHEMA_OP= 1 << 1,
   /*
     Skip drop of ndb table in delete_table.  Used when calling
     mysql_rm_table_part2 in "show tables", as we do not want to
     remove ndb tables "by mistake".  The table should not exist
     in ndb in the first place.
   */
-  ,TNO_NO_NDB_DROP_TABLE=    1 << 2
+  TNO_NO_NDB_DROP_TABLE=    1 << 2,
+  /*
+    In participating mysqld, do not remove stray files
+    when dropping tables
+  */
+  TNO_NO_REMOVE_STRAY_FILES= 1 <<3
 };
 
 enum THD_NDB_TRANS_OPTIONS
@@ -338,11 +304,10 @@ class Thd_ndb
   bool m_force_send;
 
   int m_error_code;
-  uint32 m_query_id; /* query id whn m_error_code was set */
+  query_id_t m_query_id; /* query id whn m_error_code was set */
   uint32 options;
   uint32 trans_options;
   List<NDB_SHARE> changed_tables;
-  uint query_state;
   HASH open_tables;
   /*
     This is a memroot used to buffer rows for batched execution.
@@ -357,12 +322,12 @@ class Thd_ndb
   uint m_batch_size;
 
   uint m_execute_count;
-  uint m_max_violation_count;
-  uint m_old_violation_count;
-  uint m_conflict_fn_usage_count;
 
   uint m_scan_count;
   uint m_pruned_scan_count;
+  
+  uint m_transaction_no_hint_count[MAX_NDB_NODES];
+  uint m_transaction_hint_count[MAX_NDB_NODES];
 
   NdbTransaction *global_schema_lock_trans;
   uint global_schema_lock_count;
@@ -373,6 +338,23 @@ class Thd_ndb
   bool recycle_ndb(THD* thd);
 };
 
+struct st_ndb_status {
+  st_ndb_status() { bzero(this, sizeof(struct st_ndb_status)); }
+  long cluster_node_id;
+  const char * connected_host;
+  long connected_port;
+  long number_of_replicas;
+  long number_of_data_nodes;
+  long number_of_ready_data_nodes;
+  long connect_count;
+  long execute_count;
+  long scan_count;
+  long pruned_scan_count;
+  long transaction_no_hint_count[MAX_NDB_NODES];
+  long transaction_hint_count[MAX_NDB_NODES];
+  long long api_client_stats[Ndb::NumClientStatistics];
+};
+
 int ndbcluster_commit(handlerton *hton, THD *thd, bool all);
 class ha_ndbcluster: public handler
 {
@@ -380,14 +362,16 @@ class ha_ndbcluster: public handler
   ha_ndbcluster(handlerton *hton, TABLE_SHARE *table);
   ~ha_ndbcluster();
 
-  int ha_initialise();
+#ifndef NDB_WITHOUT_READ_BEFORE_WRITE_REMOVAL
   void column_bitmaps_signal(uint sig_type);
+#endif
   int open(const char *name, int mode, uint test_if_locked);
   int close(void);
   void local_close(THD *thd, bool release_metadata);
 
   int optimize(THD* thd, HA_CHECK_OPT* check_opt);
   int analyze(THD* thd, HA_CHECK_OPT* check_opt);
+  int analyze_index(THD* thd);
 
   int write_row(uchar *buf);
   int update_row(const uchar *old_data, uchar *new_data);
@@ -415,8 +399,8 @@ class ha_ndbcluster: public handler
                               bool eq_range, bool sorted,
                               uchar* buf);
   int read_range_next();
-  int alter_tablespace(st_alter_tablespace *info);
 
+#ifndef NDB_WITH_NEW_MRR_INTERFACE
   /**
    * Multi range stuff
    */
@@ -427,16 +411,20 @@ class ha_ndbcluster: public handler
   bool null_value_index_search(KEY_MULTI_RANGE *ranges,
 			       KEY_MULTI_RANGE *end_range,
 			       HANDLER_BUFFER *buffer);
+#endif
 
   bool get_error_message(int error, String *buf);
   ha_rows records();
   ha_rows estimate_rows_upper_bound()
     { return HA_POS_ERROR; }
   int info(uint);
-  void get_dynamic_partition_info(PARTITION_INFO *stat_info, uint part_id);
+#if MYSQL_VERSION_ID < 50501
+  typedef PARTITION_INFO PARTITION_STATS;
+#endif
+  void get_dynamic_partition_info(PARTITION_STATS *stat_info, uint part_id);
   uint32 calculate_key_hash_value(Field **field_array);
-  bool read_before_write_removal_possible(List<Item> *fields,
-                                          List<Item> *values);
+  bool read_before_write_removal_possible();
+  ha_rows read_before_write_removal_rows_written(void) const;
   int extra(enum ha_extra_function operation);
   int extra_opt(enum ha_extra_function operation, ulong cache_size);
   int reset();
@@ -496,7 +484,7 @@ class ha_ndbcluster: public handler
   int ndb_update_row(const uchar *old_data, uchar *new_data,
                      int is_bulk_update);
 
-  static Thd_ndb* seize_thd_ndb();
+  static Thd_ndb* seize_thd_ndb(THD*);
   static void release_thd_ndb(Thd_ndb* thd_ndb);
  
 static void set_dbname(const char *pathname, char *dbname);
@@ -538,7 +526,7 @@ static void set_tabname(const char *pathname, char *tabname);
    =, !=, >, >=, <, <=, like, "not like", "is null", and "is not null". 
    Negated conditions are supported by NOT which generate NAND/NOR groups.
  */ 
-  const COND *cond_push(const COND *cond);
+  const Item *cond_push(const Item *cond);
  /*
    Pop the top condition from the condition stack of the handler instance.
    SYNOPSIS
@@ -559,8 +547,10 @@ static void set_tabname(const char *pathname, char *tabname);
                                      qc_engine_callback *engine_callback,
                                      ulonglong *engine_data);
 
+#ifndef NDB_WITHOUT_ONLINE_ALTER
   int check_if_supported_alter(TABLE *altered_table,
                                HA_CREATE_INFO *create_info,
+                               HA_ALTER_INFO *alter_info,
                                HA_ALTER_FLAGS *alter_flags,
                                uint table_changes);
 
@@ -580,23 +570,18 @@ static void set_tabname(const char *pathname, char *tabname);
                          HA_CREATE_INFO *create_info,
                          HA_ALTER_INFO *alter_info,
                          HA_ALTER_FLAGS *alter_flags);
+#endif
 
 private:
 #ifdef HAVE_NDB_BINLOG
-  int delete_row_conflict_fn(enum_conflict_fn_type cft,
-                             const uchar *old_data,
-                             NdbInterpretedCode *);
-  int write_row_conflict_fn(enum_conflict_fn_type cft,
-                            uchar *data,
-                            NdbInterpretedCode *);
-  int update_row_conflict_fn(enum_conflict_fn_type cft,
-                             const uchar *old_data,
-                             uchar *new_data,
-                             NdbInterpretedCode *);
-  int row_conflict_fn_max(const uchar *new_data,
-                          NdbInterpretedCode *);
-  int row_conflict_fn_old(const uchar *old_data,
-                          NdbInterpretedCode *);
+  int prepare_conflict_detection(enum_conflicting_op_type op_type,
+                                 const NdbRecord* key_rec,
+                                 const uchar* old_data,
+                                 const uchar* new_data,
+                                 NdbTransaction* trans,
+                                 NdbInterpretedCode* code,
+                                 NdbOperation::OperationOptions* options,
+                                 bool& conflict_handled);
 #endif
   void setup_key_ref_for_ndb_record(const NdbRecord **key_rec,
                                     const uchar **key_row,
@@ -631,6 +616,7 @@ private:
   int add_hidden_pk_ndb_record(NdbDictionary::Dictionary *dict);
   int add_index_ndb_record(NdbDictionary::Dictionary *dict,
                            KEY *key_info, uint index_no);
+  int check_default_values(const NdbDictionary::Table* ndbtab);
   int get_metadata(THD *thd, const char* path);
   void release_metadata(THD *thd, Ndb *ndb);
   NDB_INDEX_TYPE get_index_type(uint idx_no) const;
@@ -640,15 +626,16 @@ private:
   bool has_null_in_unique_index(uint idx_no) const;
   bool check_index_fields_not_null(KEY *key_info);
 
-  uint set_up_partition_info(partition_info *part_info,
-                             TABLE *table,
-                             void *tab);
-  int set_range_data(void *tab, partition_info* part_info);
-  int set_list_data(void *tab, partition_info* part_info);
+  int set_up_partition_info(partition_info *part_info,
+                            NdbDictionary::Table&) const;
+  int set_range_data(const partition_info* part_info,
+                     NdbDictionary::Table&) const;
+  int set_list_data(const partition_info* part_info,
+                    NdbDictionary::Table&) const;
   int ndb_pk_update_row(THD *thd, 
                         const uchar *old_data, uchar *new_data,
                         uint32 old_part_id);
-  int pk_read(const uchar *key, uint key_len, uchar *buf, uint32 part_id);
+  int pk_read(const uchar *key, uint key_len, uchar *buf, uint32 *part_id);
   int ordered_index_scan(const key_range *start_key,
                          const key_range *end_key,
                          bool sorted, bool descending, uchar* buf,
@@ -664,14 +651,14 @@ private:
                     bool batched_update);
 
   bool start_bulk_delete();
-  int bulk_delete_row(const uchar *record);
   int end_bulk_delete();
-  int ndb_delete_row(const uchar *record, bool primary_key_update,
-                     bool is_bulk_delete= FALSE);
+  int ndb_delete_row(const uchar *record, bool primary_key_update);
 
   int ndb_optimize_table(THD* thd, uint delay);
 
+#ifndef NDB_WITHOUT_ONLINE_ALTER
   int alter_frm(THD *thd, const char *file, NDB_ALTER_DATA *alter_data);
+#endif
 
   bool check_all_operations_for_error(NdbTransaction *trans,
                                       const NdbOperation *first,
@@ -685,8 +672,6 @@ private:
   int next_result(uchar *buf); 
   int close_scan();
   void unpack_record(uchar *dst_row, const uchar *src_row);
-  int get_ndb_lock_type(enum thr_lock_type type,
-                        const MY_BITMAP *column_bitmap);
 
   void set_dbname(const char *pathname);
   void set_tabname(const char *pathname);
@@ -717,7 +702,8 @@ private:
                                NdbOperation::GetValueSpec gets[2]);
   void get_hidden_fields_scan(NdbScanOperation::ScanOptions *options,
                               NdbOperation::GetValueSpec gets[2]);
-  void eventSetAnyValue(THD *thd, NdbOperation::OperationOptions *options);
+  void eventSetAnyValue(THD *thd,
+                        NdbOperation::OperationOptions *options) const;
   bool check_index_fields_in_write_set(uint keyno);
 
   const NdbOperation *pk_unique_index_read_key(uint idx, 
@@ -733,7 +719,7 @@ private:
                                   ulonglong nb_desired_values,
                                   ulonglong *first_value,
                                   ulonglong *nb_reserved_values);
-  bool uses_blob_value(const MY_BITMAP *bitmap);
+  bool uses_blob_value(const MY_BITMAP *bitmap) const;
 
   static inline bool isManualBinlogExec(THD *thd);
 
@@ -749,6 +735,20 @@ private:
   void no_uncommitted_rows_update(int);
   void no_uncommitted_rows_reset(THD *);
 
+  /* Ordered index statistics v4 */
+  int ndb_index_stat_query(uint inx,
+                           const key_range *min_key,
+                           const key_range *max_key,
+                           NdbIndexStat::Stat& stat,
+                           int from);
+  int ndb_index_stat_get_rir(uint inx,
+                             key_range *min_key,
+                             key_range *max_key,
+                             ha_rows *rows_out);
+  int ndb_index_stat_set_rpk(uint inx);
+  int ndb_index_stat_analyze(Ndb *ndb,
+                             uint *inx_list,
+                             uint inx_count);
 
   NdbTransaction *start_transaction_part_id(uint32 part_id, int &error);
   inline NdbTransaction *get_transaction_part_id(uint32 part_id, int &error)
@@ -766,6 +766,13 @@ private:
     return start_transaction(error);
   }
 
+  NdbTransaction *start_transaction_row(const NdbRecord *ndb_record,
+                                        const uchar *record,
+                                        int &error);
+  NdbTransaction *start_transaction_key(uint index,
+                                        const uchar *key_data,
+                                        int &error);
+
   friend int check_completed_operations_pre_commit(Thd_ndb*,
                                                    NdbTransaction*,
                                                    const NdbOperation*,
@@ -800,7 +807,6 @@ private:
   char m_dbname[FN_HEADLEN];
   //char m_schemaname[FN_HEADLEN];
   char m_tabname[FN_HEADLEN];
-  ulonglong m_table_flags;
   THR_LOCK_DATA m_lock;
   bool m_lock_tuple;
   NDB_SHARE *m_share;
@@ -845,6 +851,7 @@ private:
   bool m_skip_auto_increment;
   bool m_blobs_pending;
   bool m_slow_path;
+  bool m_is_bulk_delete;
 
   /* State for setActiveHook() callback for reading blob data. */
   uint m_blob_counter;
@@ -854,7 +861,7 @@ private:
   
   // memory for blobs in one tuple
   uchar *m_blobs_buffer;
-  uint32 m_blobs_buffer_size;
+  Uint64 m_blobs_buffer_size;
   uint m_dupkey;
   // set from thread variables at external lock
   ha_rows m_autoincrement_prefetch;
@@ -872,15 +879,13 @@ private:
   NdbIndexScanOperation *m_multi_cursor;
   Ndb *get_ndb(THD *thd);
 
-  int update_stats(THD *thd, bool do_read_stat, bool have_lock= FALSE);
+  int update_stats(THD *thd, bool do_read_stat, bool have_lock= FALSE,
+                   uint part_id= ~(uint)0);
+  int add_handler_to_open_tables(THD*, Thd_ndb*, ha_ndbcluster* handler);
 };
 
-extern SHOW_VAR ndb_status_variables[];
-
 int ndbcluster_discover(THD* thd, const char* dbname, const char* name,
                         const void** frmblob, uint* frmlen);
-int ndbcluster_find_files(THD *thd,const char *db,const char *path,
-                          const char *wild, bool dir, List<LEX_STRING> *files);
 int ndbcluster_table_exists_in_engine(THD* thd,
                                       const char *db, const char *name);
 void ndbcluster_print_error(int error, const NdbOperation *error_op);
@@ -888,5 +893,9 @@ void ndbcluster_print_error(int error, const NdbOperation *error_op);
 static const char ndbcluster_hton_name[]= "ndbcluster";
 static const int ndbcluster_hton_name_length=sizeof(ndbcluster_hton_name)-1;
 extern int ndbcluster_terminating;
-extern int ndb_util_thread_running;
-extern pthread_cond_t COND_ndb_util_ready;
+
+#include "ndb_util_thread.h"
+extern Ndb_util_thread ndb_util_thread;
+
+#include "ha_ndb_index_stat.h"
+extern Ndb_index_stat_thread ndb_index_stat_thread;
