@@ -1124,6 +1124,9 @@ Item_in_subselect::single_value_transformer(JOIN *join,
     select_lex->having= join->having= and_items(join->having, item);
     if (join->having == item)
       item->name= (char*)in_having_cond;
+#ifndef MCP_BUG58818
+    select_lex->having->top_level_item();
+#endif
     select_lex->having_fix_field= 1;
     /*
       we do not check join->having->fixed, because Item_and (from and_items)
@@ -1136,7 +1139,7 @@ Item_in_subselect::single_value_transformer(JOIN *join,
   }
   else
   {
-    Item *item= (Item*) select_lex->item_list.head()->real_item();
+    Item *item= (Item*) select_lex->item_list.head();
 
     if (select_lex->table_list.elements)
     {
