@@ -755,9 +755,14 @@ public:
   store_key_const_item(THD *thd, Field *to_field_arg, uchar *ptr,
 		       uchar *null_ptr_arg, uint length,
 		       Item *item_arg)
+#ifndef MCP_BUG58628
+    :store_key_item(thd, to_field_arg, ptr,
+                    null_ptr_arg, length, item_arg), inited(0)
+#else
     :store_key_item(thd, to_field_arg,ptr,
-		    null_ptr_arg ? null_ptr_arg : item_arg->maybe_null ?
-		    &err : (uchar*) 0, length, item_arg), inited(0)
+                   null_ptr_arg ? null_ptr_arg : item_arg->maybe_null ?
+                   &err : (uchar*) 0, length, item_arg), inited(0)
+#endif
   {
   }
   const char *name() const { return "const"; }
@@ -765,12 +770,21 @@ public:
 protected:  
   enum store_key_result copy_inner()
   {
+#ifndef MCP_BUG58628
+    if (!inited)
+    {
+      inited=1;
+      int res= store_key_item::copy_inner();
+      if (res && !err)
+        err= res;
+    }
+#else
     int res;
     if (!inited)
     {
       inited=1;
       if ((res= item->save_in_field(to_field, 1)))
-      {       
+      {
         if (!err)
           err= res < 0 ? 1 : res; /* 1=STORE_KEY_FATAL */
       }
@@ -782,6 +796,7 @@ protected:
         err= 1; /* STORE_KEY_FATAL */
     }
     null_key= to_field->is_null() || item->null_value;
+#endif
     return (err > 2 ? STORE_KEY_FATAL : (store_key_result) err);
   }
 };
