@@ -1,4 +1,6 @@
-/* Copyright (C) 2003 MySQL AB
+/*
+   Copyright (c) 2003-2006 MySQL AB, 2008 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,24 +13,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef API_VERSION_HPP
 #define API_VERSION_HPP
 
+#include "SignalData.hpp"
+
 class ApiVersionReq {
 /**
    * Sender(s)
    */
-  friend class MgmtSrv;
-  
+  friend class MgmtSrvr;
+
   /**
    * Reciver(s)
    */
-  friend class Qmgr;  
-public:
+  friend class Qmgr;
+
+  friend bool printAPI_VERSION_REQ(FILE *, const Uint32 *, Uint32, Uint16);
+
   STATIC_CONST( SignalLength = 4 );
-  Uint32 senderRef; 
+
+  Uint32 senderRef;
   Uint32 nodeId; //api node id
   Uint32 version; // Version of API node
   Uint32 mysql_version; // MySQL version
@@ -41,17 +49,20 @@ class ApiVersionConf {
    * Sender(s)
    */
   friend class Qmgr;
-  
+
   /**
    * Reciver(s)
    */
-  friend class MgmtSrv;  
-public:
+  friend class MgmtSrvr;
+
+  friend bool printAPI_VERSION_CONF(FILE *, const Uint32 *, Uint32, Uint16);
+
   STATIC_CONST( SignalLength = 5 );
-  Uint32 senderRef; 
+
+  Uint32 senderRef;
   Uint32 nodeId; //api node id
   Uint32 version; // Version of API node
-  Uint32 inet_addr;
+  Uint32 m_inet_addr; // The bare name "inet_addr" can conflict with arpa/inet.h
   Uint32 mysql_version; // MySQL version
 };
 
