@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +19,7 @@
 #define TC_KEY_REQ_H
 
 #include "SignalData.hpp"
+#include <transporter/TransporterDefinitions.hpp>
 
 /**
  * @class TcKeyReq
@@ -27,6 +27,21 @@
  *
  * - SENDER:    API, NDBCNTR
  * - RECEIVER:  TC
+ *
+ * Short TCKEYREQ
+ * Prior to NDB 6.4.0, TCKEYREQ was always sent as a short signal train with 
+ * up to 8 words of KeyInfo and 5 words of AttrInfo in the TCKEYREQ signal, and 
+ * all other Key and AttrInfo sent in separate signal trains.  This format is 
+ * supported for non NdbRecord operations, backwards compatibility, and for 
+ * internal TCKEYREQ signals received from non-API clients.
+ * 
+ * Long TCKEYREQ
+ * From NDB 6.4.0, for NdbRecord operations the API nodes send long TCKEYREQ 
+ * signals with all KeyInfo and AttrInfo in long sections sent with the 
+ * TCKEYREQ signal.  As each section has a section length, and no Key/AttrInfo 
+ * is sent in the TCKEYREQ signal itself, the KeyLength, AttrInfoLen and 
+ * AIInTcKeyReq fields of the header are no longer required, and their bits 
+ * can be reused in future.
  */
 class TcKeyReq {
   /**
@@ -37,12 +52,14 @@ class TcKeyReq {
   /**
    * Sender(s)
    */
-  friend class Ndbcntr;      
-  friend class NdbOperation; 
+  friend class Ndbcntr;
+  friend class NdbQueryImpl;
+  friend class NdbOperation;
   friend class NdbIndexOperation;
   friend class NdbScanOperation;
   friend class NdbBlob;
   friend class DbUtil;
+  friend class Trix;
 
   /**
    * For printing
@@ -58,7 +75,16 @@ public:
   STATIC_CONST( SignalLength = 25 );
   STATIC_CONST( MaxKeyInfo = 8 );
   STATIC_CONST( MaxAttrInfo = 5 );
-  STATIC_CONST( MaxTotalAttrInfo = 0xFFFF );
+  STATIC_CONST( MaxTotalAttrInfo = ((MAX_SEND_MESSAGE_BYTESIZE / 4) - 
+                                    SignalLength ));
+
+  /**
+   * Long signal variant of TCKEYREQ
+   */
+  STATIC_CONST( KeyInfoSectionNum = 0 );
+  STATIC_CONST( AttrInfoSectionNum = 1 );
+
+  STATIC_CONST( UnlockKeyLen = 2 );
 
 private:
 
@@ -82,11 +108,19 @@ private:
     UintR apiOperationPtr;      // DATA 1
   };
   /**
-   * ATTRIBUTE INFO (attrinfo) LENGTH
-   * This is the total length of all attribute info that is sent from
-   * the application as part of this operation. 
-   * It includes all attribute info sent in possible attrinfo 
-   * signals as well as the attribute info sent in TCKEYREQ.
+   * Short TCKEYREQ only : 
+   *   ATTRIBUTE INFO (attrinfo) LENGTH
+   *   This is the total length of all attribute info that is sent from
+   *   the application as part of this operation. 
+   *   It includes all attribute info sent in possible attrinfo 
+   *   signals as well as the attribute info sent in TCKEYREQ.
+   *
+   *   (APIVERSION << 16 | ATTRINFOLEN)
+   *
+   * Long TCKEYREQ
+   *   (APIVERSION << 16)
+   *   Get AttrInfoLength from length of section 1, if present.
+   *
    */
   UintR attrLen;              // DATA 2   (also stores API Version)
   UintR tableId;              // DATA 3
@@ -131,15 +165,17 @@ private:
   static Uint8 getDirtyFlag(const UintR & requestInfo);
   static Uint8 getInterpretedFlag(const UintR & requestInfo);
   static Uint8 getDistributionKeyFlag(const UintR & requestInfo);
+  static Uint8 getViaSPJFlag(const UintR & requestInfo);
   static Uint8 getScanIndFlag(const UintR & requestInfo);
   static Uint8 getOperationType(const UintR & requestInfo);
   static Uint8 getExecuteFlag(const UintR & requestInfo);
 
   static Uint16 getKeyLength(const UintR & requestInfo);
   static Uint8  getAIInTcKeyReq(const UintR & requestInfo);
-  static Uint8  getExecutingTrigger(const UintR & requestInfo);
   static UintR  getNoDiskFlag(const UintR & requestInfo);
 
+  static UintR getCoordinatedTransactionFlag(const UintR & requestInfo);
+
   /**
    * Get:ers for scanInfo
    */
@@ -159,15 +195,27 @@ private:
   static void setDirtyFlag(UintR & requestInfo, Uint32 flag);
   static void setInterpretedFlag(UintR & requestInfo, Uint32 flag);
   static void setDistributionKeyFlag(UintR & requestInfo, Uint32 flag);
+  static void setViaSPJFlag(UintR & requestInfo, Uint32 flag);
   static void setScanIndFlag(UintR & requestInfo, Uint32 flag);
   static void setExecuteFlag(UintR & requestInfo, Uint32 flag);  
   static void setOperationType(UintR & requestInfo, Uint32 type);
   
   static void setKeyLength(UintR & requestInfo, Uint32 len);
   static void setAIInTcKeyReq(UintR & requestInfo, Uint32 len);
-  static void setExecutingTrigger(UintR & requestInfo, Uint32 flag);
   static void setNoDiskFlag(UintR & requestInfo, UintR val);
 
+  static void setReorgFlag(UintR & requestInfo, UintR val);
+  static UintR getReorgFlag(const UintR & requestInfo);
+  static void setCoordinatedTransactionFlag(UintR & requestInfo, UintR val);
+  static void setQueueOnRedoProblemFlag(UintR & requestInfo, UintR val);
+  static UintR getQueueOnRedoProblemFlag(const UintR & requestInfo);
+
+  /**
+   * Check constraints deferred
+   */
+  static UintR getDeferredConstraints(const UintR & requestInfo);
+  static void setDeferredConstraints(UintR & requestInfo, UintR val);
+
   /**
    * Set:ers for scanInfo
    */
@@ -180,23 +228,31 @@ private:
  * Request Info
  *
  a = Attr Info in TCKEYREQ - 3  Bits -> Max 7 (Bit 16-18)
+     (Short TCKEYREQ only, for long req a == 0)
  b = Distribution Key Ind  - 1  Bit 2
+ v = Via SPJ               - 1  Bit 3
  c = Commit Indicator      - 1  Bit 4
  d = Dirty Indicator       - 1  Bit 0
  e = Scan Indicator        - 1  Bit 14
- f = Execute fired trigger - 1  Bit 19 
  i = Interpreted Indicator - 1  Bit 15
  k = Key length            - 12 Bits -> Max 4095 (Bit 20 - 31)
+     (Short TCKEYREQ only, for long req use length of
+      section 0)
  o = Operation Type        - 3  Bits -> Max 7 (Bit 5-7)
  l = Execute               - 1  Bit 10
  p = Simple Indicator      - 1  Bit 8
  s = Start Indicator       - 1  Bit 11
  y = Commit Type           - 2  Bit 12-13
  n = No disk flag          - 1  Bit 1
+ r = reorg flag            - 1  Bit 19
+ x = Coordinated Tx flag   - 1  Bit 16
+ q = Queue on redo problem - 1  Bit 9
+ D = deferred constraint   - 1  Bit 17
 
            1111111111222222222233
  01234567890123456789012345678901
- dnb cooop lsyyeiaaafkkkkkkkkkkkk
+ dnb cooop lsyyeiaaarkkkkkkkkkkkk  (Short TCKEYREQ)
+ dnbvcooopqlsyyeixD r              (Long TCKEYREQ)
 */
 
 #define TCKEY_NODISK_SHIFT (1)
@@ -207,6 +263,7 @@ private:
 #define EXECUTE_SHIFT      (10)
 #define INTERPRETED_SHIFT  (15)
 #define DISTR_KEY_SHIFT    (2)
+#define VIA_SPJ_SHIFT      (3)
 #define SCAN_SHIFT         (14)
 
 #define OPERATION_SHIFT   (5)
@@ -221,7 +278,11 @@ private:
 #define COMMIT_TYPE_SHIFT  (12)
 #define COMMIT_TYPE_MASK   (3)
 
-#define EXECUTING_TRIGGER_SHIFT (19)
+#define TC_REORG_SHIFT     (19)
+#define QUEUE_ON_REDO_SHIFT (9)
+
+#define TC_COORDINATED_SHIFT (16)
+#define TC_DEFERRED_CONSTAINTS_SHIFT (17)
 
 /**
  * Scan Info
@@ -255,12 +316,13 @@ private:
 /**
  * Attr Len
  *
- n = Attrinfo length(words)   - 16 Bits -> max 65535
+ n = Attrinfo length(words)   - 16 Bits -> max 65535 (Short TCKEYREQ only)
  a = API version no           - 16 Bits -> max 65535
 
            1111111111222222222233
  01234567890123456789012345678901
- aaaaaaaaaaaaaaaannnnnnnnnnnnnnnn
+ aaaaaaaaaaaaaaaannnnnnnnnnnnnnnn   (Short TCKEYREQ)
+ aaaaaaaaaaaaaaaa                   (Long TCKEYREQ)
 */
 
 #define API_VER_NO_SHIFT     (16)
@@ -317,6 +379,18 @@ TcKeyReq::getDistributionKeyFlag(const UintR & requestInfo){
   return (Uint8)((requestInfo >> DISTR_KEY_SHIFT) & 1);
 }
 
+inline
+UintR
+TcKeyReq::getCoordinatedTransactionFlag(const UintR & requestInfo){
+  return (UintR)((requestInfo >> TC_COORDINATED_SHIFT) & 1);
+}
+
+inline
+Uint8
+TcKeyReq::getViaSPJFlag(const UintR & requestInfo){
+  return (Uint8)((requestInfo >> VIA_SPJ_SHIFT) & 1);
+}
+
 inline
 Uint8
 TcKeyReq::getScanIndFlag(const UintR & requestInfo){
@@ -341,12 +415,6 @@ TcKeyReq::getAIInTcKeyReq(const UintR & requestInfo){
   return (Uint8)((requestInfo >> AINFO_SHIFT) & AINFO_MASK);
 }
 
-inline
-Uint8
-TcKeyReq::getExecutingTrigger(const UintR & requestInfo){
-  return (Uint8)((requestInfo >> EXECUTING_TRIGGER_SHIFT) & 1);
-}
-
 inline
 void 
 TcKeyReq::clearRequestInfo(UintR & requestInfo){
@@ -417,8 +485,24 @@ TcKeyReq::setDistributionKeyFlag(UintR & requestInfo, Uint32 flag){
   requestInfo |= (flag << DISTR_KEY_SHIFT);
 }
 
+inline
+void
+TcKeyReq::setCoordinatedTransactionFlag(UintR & requestInfo, UintR flag){
+  ASSERT_BOOL(flag, "TcKeyReq::setCoordinatedTransactionFlag");
+  requestInfo &= ~(1 << TC_COORDINATED_SHIFT);
+  requestInfo |= (flag << TC_COORDINATED_SHIFT);
+}
+
 inline
 void 
+TcKeyReq::setViaSPJFlag(UintR & requestInfo, Uint32 flag){
+  ASSERT_BOOL(flag, "TcKeyReq::setViaSPJFlag");
+  requestInfo &= ~(1 << VIA_SPJ_SHIFT);
+  requestInfo |= (flag << VIA_SPJ_SHIFT);
+}
+
+inline
+void
 TcKeyReq::setScanIndFlag(UintR & requestInfo, Uint32 flag){
   ASSERT_BOOL(flag, "TcKeyReq::setScanIndFlag");
   requestInfo &= ~(1 << SCAN_SHIFT);
@@ -449,14 +533,6 @@ TcKeyReq::setAIInTcKeyReq(UintR & requestInfo, Uint32 len){
   requestInfo |= (len << AINFO_SHIFT);
 }
 
-inline
-void 
-TcKeyReq::setExecutingTrigger(UintR & requestInfo, Uint32 flag){
-  ASSERT_BOOL(flag, "TcKeyReq::setExecutingTrigger");
-  requestInfo &= ~(1 << EXECUTING_TRIGGER_SHIFT);
-  requestInfo |= (flag << EXECUTING_TRIGGER_SHIFT);
-}
-
 inline
 Uint8 
 TcKeyReq::getTakeOverScanFlag(const UintR & scanInfo){
@@ -538,4 +614,44 @@ TcKeyReq::setNoDiskFlag(UintR & requestInfo, Uint32 flag){
   requestInfo |= (flag << TCKEY_NODISK_SHIFT);
 }
 
+inline
+UintR
+TcKeyReq::getReorgFlag(const UintR & requestInfo){
+  return (requestInfo >> TC_REORG_SHIFT) & 1;
+}
+
+inline
+void
+TcKeyReq::setReorgFlag(UintR & requestInfo, Uint32 flag){
+  ASSERT_BOOL(flag, "TcKeyReq::setReorgFlag");
+  requestInfo |= (flag << TC_REORG_SHIFT);
+}
+
+inline
+UintR
+TcKeyReq::getQueueOnRedoProblemFlag(const UintR & requestInfo){
+  return (requestInfo >> QUEUE_ON_REDO_SHIFT) & 1;
+}
+
+inline
+void
+TcKeyReq::setQueueOnRedoProblemFlag(UintR & requestInfo, Uint32 flag){
+  ASSERT_BOOL(flag, "TcKeyReq::setNoDiskFlag");
+  requestInfo |= (flag << QUEUE_ON_REDO_SHIFT);
+}
+
+inline
+void
+TcKeyReq::setDeferredConstraints(UintR & requestInfo, UintR val){
+  ASSERT_BOOL(val, "TcKeyReq::setDeferredConstraints");
+  requestInfo |= (val << TC_DEFERRED_CONSTAINTS_SHIFT);
+}
+
+inline
+UintR
+TcKeyReq::getDeferredConstraints(const UintR & requestInfo){
+  return (requestInfo >> TC_DEFERRED_CONSTAINTS_SHIFT) & 1;
+}
+
+
 #endif
