@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,6 +53,10 @@ const GsnName SignalNames [] = {
   ,{ GSN_TCSEIZECONF,            "TCSEIZECONF" }
   ,{ GSN_TCSEIZEREF,             "TCSEIZEREF" }
   ,{ GSN_TCSEIZEREQ,             "TCSEIZEREQ" }
+  ,{ GSN_DBINFO_SCANREQ,         "DBINFO_SCANREQ" }
+  ,{ GSN_DBINFO_SCANCONF,        "DBINFO_SCANCONF" }
+  ,{ GSN_DBINFO_SCANREF,         "DBINFO_SCANREF" }
+  ,{ GSN_DBINFO_TRANSID_AI,      "DBINFO_TRANSID_AI" }
   ,{ GSN_TRANSID_AI,             "TRANSID_AI" }
   ,{ GSN_TRANSID_AI_R,           "TRANSID_AI_R" }
   ,{ GSN_ABORT,                  "ABORT" }
@@ -89,6 +92,7 @@ const GsnName SignalNames [] = {
   ,{ GSN_ADD_FRAGCONF,           "ADD_FRAGCONF" }
   ,{ GSN_ADD_FRAGREF,            "ADD_FRAGREF" }
   ,{ GSN_ADD_FRAGREQ,            "ADD_FRAGREQ" }
+  ,{ GSN_API_START_REP,          "API_START_REP" }
   ,{ GSN_API_FAILCONF,           "API_FAILCONF" }
   ,{ GSN_API_FAILREQ,            "API_FAILREQ" }
   ,{ GSN_CHECK_LCP_STOP,         "CHECK_LCP_STOP" }
@@ -142,9 +146,10 @@ const GsnName SignalNames [] = {
   ,{ GSN_CREATE_FRAGREF,         "CREATE_FRAGREF" }
   ,{ GSN_CREATE_FRAGREQ,         "CREATE_FRAGREQ" }
   ,{ GSN_DEBUG_SIG,              "DEBUG_SIG" }
-  ,{ GSN_DI_FCOUNTCONF,          "DI_FCOUNTCONF" }
-  ,{ GSN_DI_FCOUNTREF,           "DI_FCOUNTREF" }
-  ,{ GSN_DI_FCOUNTREQ,           "DI_FCOUNTREQ" }
+  ,{ GSN_DIH_SCAN_TAB_REQ,       "DIH_SCAN_TAB_REQ" }
+  ,{ GSN_DIH_SCAN_TAB_REF,       "DIH_SCAN_TAB_REF" }
+  ,{ GSN_DIH_SCAN_TAB_CONF,      "DIH_SCAN_TAB_CONF" }
+  ,{ GSN_DIH_SCAN_TAB_COMPLETE_REP, "DIH_SCAN_TAB_COMPLETE_REP" }
   ,{ GSN_DIADDTABCONF,           "DIADDTABCONF" }
   ,{ GSN_DIADDTABREF,            "DIADDTABREF" }
   ,{ GSN_DIADDTABREQ,            "DIADDTABREQ" }
@@ -155,26 +160,21 @@ const GsnName SignalNames [] = {
   ,{ GSN_DIGETNODESCONF,         "DIGETNODESCONF" }
   ,{ GSN_DIGETNODESREF,          "DIGETNODESREF" }
   ,{ GSN_DIGETNODESREQ,          "DIGETNODESREQ" }
-  ,{ GSN_DIGETPRIMCONF,          "DIGETPRIMCONF" }
-  ,{ GSN_DIGETPRIMREF,           "DIGETPRIMREF" }
-  ,{ GSN_DIGETPRIMREQ,           "DIGETPRIMREQ" }
+  ,{ GSN_DIH_SCAN_GET_NODES_REQ, "DIH_SCAN_GET_NODES_REQ" }
+  ,{ GSN_DIH_SCAN_GET_NODES_REF, "DIH_SCAN_GET_NODES_REF" }
+  ,{ GSN_DIH_SCAN_GET_NODES_CONF,"DIH_SCAN_GET_NODES_CONF" }
   ,{ GSN_DIH_RESTARTCONF,        "DIH_RESTARTCONF" }
   ,{ GSN_DIH_RESTARTREF,         "DIH_RESTARTREF" }
   ,{ GSN_DIH_RESTARTREQ,         "DIH_RESTARTREQ" }
 
-  ,{ GSN_DIRELEASECONF,          "DIRELEASECONF" }
-  ,{ GSN_DIRELEASEREF,           "DIRELEASEREF" }
-  ,{ GSN_DIRELEASEREQ,           "DIRELEASEREQ" }
   ,{ GSN_DISCONNECT_REP,         "DISCONNECT_REP" }
-  ,{ GSN_DISEIZECONF,            "DISEIZECONF" }
-  ,{ GSN_DISEIZEREF,             "DISEIZEREF" }
-  ,{ GSN_DISEIZEREQ,             "DISEIZEREQ" }
   ,{ GSN_DIVERIFYCONF,           "DIVERIFYCONF" }
   ,{ GSN_DIVERIFYREF,            "DIVERIFYREF" }
   ,{ GSN_DIVERIFYREQ,            "DIVERIFYREQ" }
   ,{ GSN_EMPTY_LCP_REQ,          "EMPTY_LCP_REQ" }
   ,{ GSN_EMPTY_LCP_CONF,         "EMPTY_LCP_CONF" }
-  ,{ GSN_ENABLE_COMORD,          "ENABLE_COMORD" }
+  ,{ GSN_ENABLE_COMREQ,          "ENABLE_COMREQ" }
+  ,{ GSN_ENABLE_COMCONF,         "ENABLE_COMCONF" }
   ,{ GSN_END_LCPCONF,            "END_LCPCONF" }
   ,{ GSN_END_LCPREQ,             "END_LCPREQ" }
   ,{ GSN_END_TOCONF,             "END_TOCONF" }
@@ -259,9 +259,9 @@ const GsnName SignalNames [] = {
   ,{ GSN_NEXT_SCANREF,           "NEXT_SCANREF" }
   ,{ GSN_NEXT_SCANREQ,           "NEXT_SCANREQ" }
   ,{ GSN_NF_COMPLETEREP,         "NF_COMPLETEREP" }
-  ,{ GSN_OPEN_COMCONF,           "OPEN_COMCONF" }
-  ,{ GSN_OPEN_COMREF,            "OPEN_COMREF" }
-  ,{ GSN_OPEN_COMREQ,            "OPEN_COMREQ" }
+//  ,{ GSN_OPEN_COMCONF,           "OPEN_COMCONF" }
+//  ,{ GSN_OPEN_COMREF,            "OPEN_COMREF" }
+  ,{ GSN_OPEN_COMORD,            "OPEN_COMORD" }
   ,{ GSN_PACKED_SIGNAL,          "PACKED_SIGNAL" }
   ,{ GSN_PREP_FAILCONF,          "PREP_FAILCONF" }
   ,{ GSN_PREP_FAILREF,           "PREP_FAILREF" }
@@ -428,10 +428,6 @@ const GsnName SignalNames [] = {
   ,{ GSN_PREP_DROP_TAB_REF,      "PREP_DROP_TAB_REF" }
   ,{ GSN_PREP_DROP_TAB_CONF,     "PREP_DROP_TAB_CONF" }
 
-  ,{ GSN_WAIT_DROP_TAB_REQ,      "WAIT_DROP_TAB_REQ" }
-  ,{ GSN_WAIT_DROP_TAB_REF,      "WAIT_DROP_TAB_REF" }
-  ,{ GSN_WAIT_DROP_TAB_CONF,     "WAIT_DROP_TAB_CONF" }
-  
   ,{ GSN_CREATE_TRIG_REQ,        "CREATE_TRIG_REQ" }
   ,{ GSN_CREATE_TRIG_CONF,       "CREATE_TRIG_CONF" }
   ,{ GSN_CREATE_TRIG_REF,        "CREATE_TRIG_REF" }
@@ -506,6 +502,10 @@ const GsnName SignalNames [] = {
   ,{ GSN_BACKUP_STATUS_CONF,     "BACKUP_STATUS_CONF" }
   ,{ GSN_SIGNAL_DROPPED_REP,     "SIGNAL_DROPPED_REP" }
   ,{ GSN_CONTINUE_FRAGMENTED,    "CONTINUE_FRAGMENTED" }
+  ,{ GSN_STOP_FOR_CRASH,         "STOP_FOR_CRASH" }
+  ,{ GSN_BACKUP_LOCK_TAB_REQ,    "BACKUP_LOCK_TAB_REQ" }
+  ,{ GSN_BACKUP_LOCK_TAB_CONF,   "BACKUP_LOCK_TAB_CONF" }
+  ,{ GSN_BACKUP_LOCK_TAB_REF,    "BACKUP_LOCK_TAB_REF" }
 
   /** Util Block Services **/
   ,{ GSN_UTIL_SEQUENCE_REQ,      "UTIL_SEQUENCE_REQ" }
@@ -610,34 +610,37 @@ const GsnName SignalNames [] = {
   ,{ GSN_DROP_FILE_REF,  "DROP_FILE_REF" }
   ,{ GSN_DROP_FILE_CONF, "DROP_FILE_CONF" }
   
-  ,{ GSN_CREATE_OBJ_REQ,  "CREATE_OBJ_REQ" }
-  ,{ GSN_CREATE_OBJ_REF,  "CREATE_OBJ_REF" }
-  ,{ GSN_CREATE_OBJ_CONF, "CREATE_OBJ_CONF" }
+  ,{ GSN_CREATE_FILEGROUP_IMPL_REQ, "CREATE_FILEGROUP_IMPL_REQ" }
+  ,{ GSN_CREATE_FILEGROUP_IMPL_REF, "CREATE_FILEGROUP_IMPL_REF" }
+  ,{ GSN_CREATE_FILEGROUP_IMPL_CONF, "CREATE_FILEGROUP_IMPL_CONF" }
   
-  ,{ GSN_DROP_OBJ_REQ,  "DROP_OBJ_REQ" }
-  ,{ GSN_DROP_OBJ_REF,  "DROP_OBJ_REF" }
-  ,{ GSN_DROP_OBJ_CONF, "DROP_OBJ_CONF" }
+  ,{ GSN_CREATE_FILE_IMPL_REQ,  "CREATE_FILE_IMPL_REQ" }
+  ,{ GSN_CREATE_FILE_IMPL_REF,  "CREATE_FILE_IMPL_REF" }
+  ,{ GSN_CREATE_FILE_IMPL_CONF, "CREATE_FILE_IMPL_CONF" }
+
+  ,{ GSN_DROP_FILEGROUP_IMPL_REQ,  "DROP_FILEGROUP_IMPL_REQ" }
+  ,{ GSN_DROP_FILEGROUP_IMPL_REF,  "DROP_FILEGROUP_IMPL_REF" }
+  ,{ GSN_DROP_FILEGROUP_IMPL_CONF, "DROP_FILEGROUP_IMPL_CONF" }
+
+  ,{ GSN_DROP_FILE_IMPL_REQ,  "DROP_FILE_IMPL_REQ" }
+  ,{ GSN_DROP_FILE_IMPL_REF,  "DROP_FILE_IMPL_REF" }
+  ,{ GSN_DROP_FILE_IMPL_CONF, "DROP_FILE_IMPL_CONF" }
   
   ,{ GSN_LCP_PREPARE_REQ,  "LCP_PREPARE_REQ" }
   ,{ GSN_LCP_PREPARE_REF,  "LCP_PREPARE_REF" }
   ,{ GSN_LCP_PREPARE_CONF, "LCP_PREPARE_CONF" }
 
-  ,{ GSN_DICT_ABORT_REQ,  "DICT_ABORT_REQ" }
-  ,{ GSN_DICT_ABORT_REF,  "DICT_ABORT_REF" }
-  ,{ GSN_DICT_ABORT_CONF, "DICT_ABORT_CONF" }
-
-  ,{ GSN_DICT_COMMIT_REQ,  "DICT_COMMIT_REQ" }
-  ,{ GSN_DICT_COMMIT_REF,  "DICT_COMMIT_REF" }
-  ,{ GSN_DICT_COMMIT_CONF, "DICT_COMMIT_CONF" }
-
   /* DICT LOCK */
   ,{ GSN_DICT_LOCK_REQ,          "DICT_LOCK_REQ" }
   ,{ GSN_DICT_LOCK_CONF,         "DICT_LOCK_CONF" }
   ,{ GSN_DICT_LOCK_REF,          "DICT_LOCK_REF" }
   ,{ GSN_DICT_UNLOCK_ORD,        "DICT_UNLOCK_ORD" }
 
+  ,{ GSN_DICT_TAKEOVER_REQ,  "DICT_TAKEOVER_REQ" }
+  ,{ GSN_DICT_TAKEOVER_REF,  "DICT_TAKEOVER_REF" }
+  ,{ GSN_DICT_TAKEOVER_CONF, "DICT_TAKEOVER_CONF" }
+
   ,{ GSN_UPDATE_FRAG_DIST_KEY_ORD, "UPDATE_FRAG_DIST_KEY_ORD" }
-  ,{ GSN_DICT_COMMIT_REQ,  "DICT_COMMIT_REQ"}
 
   ,{ GSN_ROUTE_ORD, "ROUTE_ORD" }
   ,{ GSN_NODE_VERSION_REP, "NODE_VERSION_REP" }
@@ -654,10 +657,127 @@ const GsnName SignalNames [] = {
   ,{ GSN_END_TOREF, "END_TOREF" }
   ,{ GSN_START_PERMREP, "START_PERMREP" }
 
+  ,{ GSN_SCHEMA_TRANS_BEGIN_REQ, "SCHEMA_TRANS_BEGIN_REQ" }
+  ,{ GSN_SCHEMA_TRANS_BEGIN_CONF, "SCHEMA_TRANS_BEGIN_CONF" }
+  ,{ GSN_SCHEMA_TRANS_BEGIN_REF, "SCHEMA_TRANS_BEGIN_REF" }
+  ,{ GSN_SCHEMA_TRANS_END_REQ, "SCHEMA_TRANS_END_REQ" }
+  ,{ GSN_SCHEMA_TRANS_END_CONF, "SCHEMA_TRANS_END_CONF" }
+  ,{ GSN_SCHEMA_TRANS_END_REF, "SCHEMA_TRANS_END_REF" }
+  ,{ GSN_SCHEMA_TRANS_END_REP, "SCHEMA_TRANS_END_REP" }
+  ,{ GSN_SCHEMA_TRANS_IMPL_REQ, "SCHEMA_TRANS_IMPL_REQ" }
+  ,{ GSN_SCHEMA_TRANS_IMPL_CONF, "SCHEMA_TRANS_IMPL_CONF" }
+  ,{ GSN_SCHEMA_TRANS_IMPL_REF, "SCHEMA_TRANS_IMPL_REF" }
+
+  ,{ GSN_CREATE_TRIG_IMPL_REQ, "CREATE_TRIG_IMPL_REQ" }
+  ,{ GSN_CREATE_TRIG_IMPL_CONF, "CREATE_TRIG_IMPL_CONF" }
+  ,{ GSN_CREATE_TRIG_IMPL_REF, "CREATE_TRIG_IMPL_REF" }
+  ,{ GSN_DROP_TRIG_IMPL_REQ, "DROP_TRIG_IMPL_REQ" }
+  ,{ GSN_DROP_TRIG_IMPL_CONF, "DROP_TRIG_IMPL_CONF" }
+  ,{ GSN_DROP_TRIG_IMPL_REF, "DROP_TRIG_IMPL_REF" }
+  ,{ GSN_ALTER_TRIG_IMPL_REQ, "ALTER_TRIG_IMPL_REQ" }
+  ,{ GSN_ALTER_TRIG_IMPL_CONF, "ALTER_TRIG_IMPL_CONF" }
+  ,{ GSN_ALTER_TRIG_IMPL_REF, "ALTER_TRIG_IMPL_REF" }
+
+  ,{ GSN_CREATE_INDX_IMPL_REQ, "CREATE_INDX_IMPL_REQ" }
+  ,{ GSN_CREATE_INDX_IMPL_CONF, "CREATE_INDX_IMPL_CONF" }
+  ,{ GSN_CREATE_INDX_IMPL_REF, "CREATE_INDX_IMPL_REF" }
+  ,{ GSN_DROP_INDX_IMPL_REQ, "DROP_INDX_IMPL_REQ" }
+  ,{ GSN_DROP_INDX_IMPL_CONF, "DROP_INDX_IMPL_CONF" }
+  ,{ GSN_DROP_INDX_IMPL_REF, "DROP_INDX_IMPL_REF" }
+  ,{ GSN_ALTER_INDX_IMPL_REQ, "ALTER_INDX_IMPL_REQ" }
+  ,{ GSN_ALTER_INDX_IMPL_CONF, "ALTER_INDX_IMPL_CONF" }
+  ,{ GSN_ALTER_INDX_IMPL_REF, "ALTER_INDX_IMPL_REF" }
+
+  ,{ GSN_DROP_FRAG_REQ,  "DROP_FRAG_REQ" }
+  ,{ GSN_DROP_FRAG_REF,  "DROP_FRAG_REF" }
+  ,{ GSN_DROP_FRAG_CONF, "DROP_FRAG_CONF" }
+
+  ,{ GSN_BUILD_INDX_IMPL_REQ, "BUILD_INDX_IMPL_REQ" }
+  ,{ GSN_BUILD_INDX_IMPL_CONF, "BUILD_INDX_IMPL_CONF" }
+  ,{ GSN_BUILD_INDX_IMPL_REF, "BUILD_INDX_IMPL_REF" }
+
+  ,{ GSN_RESTORE_LCP_REQ, "RESTORE_LCP_REQ" }
+  ,{ GSN_RESTORE_LCP_CONF, "RESTORE_LCP_CONF" }
+  ,{ GSN_RESTORE_LCP_REF, "RESTORE_LCP_REF" }
+
+  ,{ GSN_CREATE_NODEGROUP_REQ, "CREATE_NODEGROUP_REQ" }
+  ,{ GSN_CREATE_NODEGROUP_CONF, "CREATE_NODEGROUP_CONF" }
+  ,{ GSN_CREATE_NODEGROUP_REF, "CREATE_NODEGROUP_REF" }
+
+  ,{ GSN_CREATE_NODEGROUP_IMPL_REQ, "CREATE_NODEGROUP_IMPL_REQ" }
+  ,{ GSN_CREATE_NODEGROUP_IMPL_CONF, "CREATE_NODEGROUP_IMPL_CONF" }
+  ,{ GSN_CREATE_NODEGROUP_IMPL_REF, "CREATE_NODEGROUP_IMPL_REF" }
+
+  ,{ GSN_DROP_NODEGROUP_REQ, "DROP_NODEGROUP_REQ" }
+  ,{ GSN_DROP_NODEGROUP_CONF, "DROP_NODEGROUP_CONF" }
+  ,{ GSN_DROP_NODEGROUP_REF, "DROP_NODEGROUP_REF" }
+
+  ,{ GSN_DROP_NODEGROUP_IMPL_REQ, "DROP_NODEGROUP_IMPL_REQ" }
+  ,{ GSN_DROP_NODEGROUP_IMPL_CONF, "DROP_NODEGROUP_IMPL_CONF" }
+  ,{ GSN_DROP_NODEGROUP_IMPL_REF, "DROP_NODEGROUP_IMPL_REF" }
+
+  ,{ GSN_CONFIG_CHECK_REQ, "CONFIG_CHECK_REQ" }
+  ,{ GSN_CONFIG_CHECK_REF, "CONFIG_CHECK_REF" }
+  ,{ GSN_CONFIG_CHECK_CONF, "CONFIG_CHECK_CONF" }
+
+  ,{ GSN_CONFIG_CHANGE_REQ, "CONFIG_CHANGE_REQ" }
+  ,{ GSN_CONFIG_CHANGE_REF, "CONFIG_CHANGE_REF" }
+  ,{ GSN_CONFIG_CHANGE_CONF, "CONFIG_CHANGE_CONF" }
+
+  ,{ GSN_CONFIG_CHANGE_IMPL_REQ, "CONFIG_CHANGE_IMPL_REQ" }
+  ,{ GSN_CONFIG_CHANGE_IMPL_REF, "CONFIG_CHANGE_IMPL_REF" }
+  ,{ GSN_CONFIG_CHANGE_IMPL_CONF, "CONFIG_CHANGE_IMPL_CONF" }
+
+  ,{ GSN_DATA_FILE_ORD, "DATA_FILE_ORD" }
+
+  ,{ GSN_CALLBACK_REQ, "CALLBACK_REQ" }
+  ,{ GSN_CALLBACK_CONF, "CALLBACK_CONF" }
+  ,{ GSN_CALLBACK_ACK, "CALLBACK_ACK" }
+
+  ,{ GSN_RELEASE_PAGES_REQ, "RELEASE_PAGES_REQ" }
+  ,{ GSN_RELEASE_PAGES_CONF, "RELEASE_PAGES_CONF" }
+
+  ,{ GSN_CREATE_HASH_MAP_REQ, "CREATE_HASH_MAP_REQ" }
+  ,{ GSN_CREATE_HASH_MAP_REF, "CREATE_HASH_MAP_REF" }
+  ,{ GSN_CREATE_HASH_MAP_CONF, "CREATE_HASH_MAP_CONF" }
+
+  ,{ GSN_SYNC_THREAD_REQ, "SYNC_THREAD_REQ" }
+  ,{ GSN_SYNC_THREAD_CONF, "SYNC_THREAD_CONF" }
+
   ,{ GSN_LOCAL_ROUTE_ORD, "LOCAL_ROUTE_ORD" }
 
   ,{ GSN_ALLOC_MEM_REQ, "ALLOC_MEM_REQ" }
   ,{ GSN_ALLOC_MEM_REF, "ALLOC_MEM_REF" }
   ,{ GSN_ALLOC_MEM_CONF, "ALLOC_MEM_CONF" }
+
+  ,{ GSN_DIH_GET_TABINFO_REQ, "DIH_GET_TABINFO_REQ" }
+  ,{ GSN_DIH_GET_TABINFO_REF, "DIH_GET_TABINFO_REF" }
+  ,{ GSN_DIH_GET_TABINFO_CONF, "DIH_GET_TABINFO_CONF" }
+
+  ,{ GSN_SYNC_REQ, "SYNC_REQ" }
+  ,{ GSN_SYNC_REF, "SYNC_REF" }
+  ,{ GSN_SYNC_CONF, "SYNC_CONF" }
+
+  ,{ GSN_SYNC_PATH_REQ, "SYNC_PATH_REQ" }
+  ,{ GSN_SYNC_PATH_CONF, "SYNC_PATH_CONF" }
+
+  ,{ GSN_NODE_PING_REQ, "NODE_PING_REQ" }
+  ,{ GSN_NODE_PING_CONF, "NODE_PING_CONF" }
+
+  ,{ GSN_INDEX_STAT_REQ, "INDEX_STAT_REQ" }
+  ,{ GSN_INDEX_STAT_CONF, "INDEX_STAT_CONF" }
+  ,{ GSN_INDEX_STAT_REF, "INDEX_STAT_REF" }
+  ,{ GSN_INDEX_STAT_IMPL_REQ, "INDEX_STAT_IMPL_REQ" }
+  ,{ GSN_INDEX_STAT_IMPL_CONF, "INDEX_STAT_IMPL_CONF" }
+  ,{ GSN_INDEX_STAT_IMPL_REF, "INDEX_STAT_IMPL_REF" }
+  ,{ GSN_INDEX_STAT_REP, "INDEX_STAT_REP" }
+
+  ,{ GSN_GET_CONFIG_REQ, "GET_CONFIG_REQ" }
+  ,{ GSN_GET_CONFIG_REF, "GET_CONFIG_REF" }
+  ,{ GSN_GET_CONFIG_CONF, "GET_CONFIG_CONF" }
+
+  ,{ GSN_LCP_STATUS_REQ, "LCP_STATUS_REQ" }
+  ,{ GSN_LCP_STATUS_CONF, "LCP_STATUS_CONF" }
+  ,{ GSN_LCP_STATUS_REF, "LCP_STATUS_REF" }
 };
 const unsigned short NO_OF_SIGNAL_NAMES = sizeof(SignalNames)/sizeof(GsnName);
